function [ solution, S, E ] = analyzeBivOfBm_BB( Y, type, interv, param, prec)
% FUNCTION [ thetahat ] = analyzeBivOfBm_BB( Y, type, interv, param, prec )
%
% Parametric estimation of bivariate OfBm (or OfGn).
%
% INPUT Y: Bivariate signal (2 lines vector). 
%
% INPUT type={'ofbm','ofgn'}  whether 'Y' should be consider as OfBm or OfGn.
%   note: if type='ofgn', the user may fractionnaly integrate the wavelet.
%   coefficients by specifying a value for param.mu_wav. (Default: 'ofbm').
%
% INPUT interv: Search interval for each variable.
%   - interv.h1 (0<=...<=1). (Default: [0 1]).
%   - interv.h2 (0<=...<=1). (Default: [0 1]).
%   - interv.beta (-1<=...<=1). (Default: [-1 1]).
%   - interv.gamma (-1<=...<=1). (Default: [-1 1]).
%   - interv.rho (0<=...<=1). (Default: [0 1]).
%   - interv.sigma1 (0<...<Inf). (Default: [0 sigma_max], see article).
%   - interv.sigma2 (0<...<Inf). (Default: [0 sigma_max], see article).
%   note: if a variable is known, enter the corresponding value.
%
% INPUT param: Algorithm parameters.
%   - param.j1. Lowest scale 2^(j1). (Default: 1).
%   - param.j2. Highest scale 2^(j2). (Default: floor(log2(length(Y))-3).
%   - param.dispAlg = {false,true}. If true, plots the progress of the algorithm AND permits to stop at any time. (Default: false).
%   - param.dispFig (0<=...<Inf). If non-zero, display wavelet spectrums in Figure '...'. (Default: 0).
%   - param.dispSol (0<=...<Inf). If non-zero, display the best current solution every '...' iterations. (Default: 0).
%   - param.crossTerm = {false,true}. If true, include the cross wavelet spectrum in the functional to minimize. (Default: true).
%   - param.divisionStep (1<...<Inf). Parameter related to the approximation of the search space by an inner convex set (see article). (Default: 50).
%   - param.itelim [Optional]. Maximum number of iterations. (Default: Inf).
%   - param.EPS [Optional]. Minimum relative bounds difference to reach. (Default: 0).
%   Only if type = 'ofgn'
%       - param.mu_wav (0<=...<=1). Fractional integration order of wavelet coefficients. (Default: 0).
%       - param.mu_bb  (0<=...<=1). Fractional integration order of wavelet coefficients to be considered in the parametric wavelet spectrum. (Default: 0 if 'param.mu_wav' = 0 / 1 otherwise).
%       note 1: having 'param.mu_bb' = 'param.mu_wav' = 0 permits to analyze 'Y' as an OfGn.
%       note 2: having 'param.mu_bb' = 'param.mu_wav' > 0 permits to analyze 'Y' as an OfGn pseudo-integrated of fractional order 'param.mu_wav'.
%       note 3: having 'param.mu_bb' > 'param.mu_wav'     permits to shift (h1,h2) by a factor ('param.mu_wav'-'param.mu_bb') in order rescale (h1,h2) into [0 1].
%
% INPUT prec: Maximum precision expected for each variable.
%   - prec.h1. (Default: 0.05).
%   - prec.h2. (Default: 0.05).
%   - prec.beta. (Default: 0.1).
%   - prec.gamma. (Default: 0.1).
%   - prec.rho. (Default: 0.1).
%   - prec.sigma1. (Default: 1).
%   - prec.sigma2. (Default: 1).
%
% OUTPUT solution: Best candidate of the global optimum at the target precision.
%
% OUTPUT S: Empirical wavelet spectrum of Y
%
% OUTPUT E: Parametric wavelet spectrum of Y obtained from 'solution'
%
% For more details, see
% J. Frecon, G. Didier, N. Pustelnik, and P. Abry, Non-Linear Wavelet Regression and Branch and Bound Optimization for the Full Identification of Bivariate Operator Fractional Brownian Motion, Submitted, 2015.
%
% note 4: wavelet coefficients are computed using Daubechies wavelets with 2 vanishing moments by means of 'DxLx1d.m' (H. Wendt, May-2007) and 'rlistcoefdaub.m' (P. Abry, 25-August-1995).
% note 5: enclosed tabulated values of eta (see article) are computed from scale 1 to 15. If it is not enough, run the file 'functions/etah/tabulate_etah.m'
%
% J. Frecon. 
% Version: 09-November-2016.



%% Default values

if nargin <= 4
    prec = struct;
end

if nargin <= 3
    param = struct;
end

if nargin == 2
    interv = struct;
end

if nargin == 1
    type = 'ofbm';
end

if strcmpi(type,'ofbm')
    sigmamax = sqrt(sum(var(diff(Y,[],2),[],2)));
else
    sigmamax = sqrt(sum(var(Y,[],2)));
end


% Interv
if ~isfield(interv,'h1')
    interv.h1 = [0 1];
end

if ~isfield(interv,'h2')
    interv.h2 = [0 1];
end

if ~isfield(interv,'beta')
    interv.beta = [-1 1];
end

if ~isfield(interv,'gamma')
    interv.gamma = [-1 1];
end

if ~isfield(interv,'rho')
    interv.rho = [0 1];
end

if ~isfield(interv,'sigma1')
    interv.sigma1 = [0 sigmamax];
end

if ~isfield(interv,'sigma2')
    interv.sigma2 = [0 sigmamax];
end


% Param
if ~isfield(param,'j1')
    param.j1 = 1;
end

if ~isfield(param,'j2')
    param.j2 = floor(log2(length(Y)))-3;
end

if ~isfield(param,'dispAlg')
    param.dispAlg = false;
end

if ~isfield(param,'dispFig')
    param.dispFig = 0;
end

if ~isfield(param,'dispSol')
    param.dispSol = 0;
end

if ~isfield(param,'crossTerm')
    param.crossTerm=true;
end

if ~isfield(param,'divisionStep')
    param.divisionStep=50;
end

if ~isfield(param,'itelim')
    param.itelim = Inf;
end

if ~isfield(param,'EPS')
    param.EPS = 0;
end

if strcmpi(type,'ofbm')
    param.dataType = 'ofbm';
else
    param.dataType = 'ofgn';
end

% Prec
if ~isfield(prec,'h1')
    prec.h1 = 0.05;
end

if ~isfield(prec,'h2')
    prec.h2 = 0.05;
end

if ~isfield(prec,'beta')
    prec.beta = 0.1;
end

if ~isfield(prec,'gamma')
    prec.gamma = 0.1;
end

if ~isfield(prec,'rho')
    prec.rho = 0.1;
end

if ~isfield(prec,'sigma1')
    prec.sigma1 = 1;
end

if ~isfield(prec,'sigma2')
    prec.sigma2 = 1;
end



% mu_wav & mu_bb
if strcmpi(param.dataType,'ofbm')
   param.mu_wav    = 0;
   param.mu_bb     = 0;
else
    
    if ~isfield(param,'mu_wav');
        param.mu_wav = 0;
    end
    
    if ~isfield(param,'mu_bb')
        if param.mu_wav == 0;
            param.mu_bb = 0;
        else
            param.mu_bb = 1;
        end
    end

end



%% Structure functions

% Wavelets coefficients
Nwt             = 2;
j1              = param.j1;
j2              = param.j2;
[cy1,~, ~]      = DxLx1d(Y(1,:),Nwt,param.mu_wav+1/2);
[cy2,~, nj]     = DxLx1d(Y(2,:),Nwt,param.mu_wav+1/2);
NJ              = nj.W;


% Wavelet spectrum
Sy11 = zeros(1,length(NJ));
Sy12 = zeros(1,length(NJ));
Sy22 = zeros(1,length(NJ));
for j=1:length(NJ);
    Sy11(j)     = mean(abs(cy1(j).value).^2);
    Sy22(j)     = mean(abs(cy2(j).value).^2);
    Sy12(j)     = mean(real(cy1(j).value_noabs.*cy2(j).value_noabs));
end
Sy11 = Sy11(j1:j2);
Sy12 = Sy12(j1:j2);
Sy22 = Sy22(j1:j2);



%% Main Algorithm
[result,nodes,remainingNodes,param]  = analyzeBivOfBm_BB_aux( Sy11, Sy12, Sy22, j1:j2, interv, param, prec);

if ~param.flag
    disp('** Current solution:')
    bb_dispSol(result)
    prompt = 'Would you like to resume [y/n]? ';
    str = input(prompt,'s');
    
    if strcmp(str,'y')
        flag = true;
    else
        flag = false;
    end
    
    while flag
        
        
        [result,nodes,remainingNodes,param]  = analyzeBivOfBm_BB_aux( Sy11, Sy12, Sy22, j1:j2, interv, param, prec, remainingNodes);
        disp('** Current solution:')
        bb_dispSol(result)
        
        if ~param.flag
            prompt = 'Would you like to resume [y/n]? ';
            str = input(prompt,'s');
            
            if strcmp(str,'y')
                flag = true;
            else
                flag = false;
            end
        else
            flag = true;
        end
        
    end
     
end



%% Termination
solution.h1     = ia_mid(result.interv.h1);
solution.h2     = ia_mid(result.interv.h2);
solution.sigma1 = ia_mid(result.interv.sigma1);
solution.sigma2 = ia_mid(result.interv.sigma2);
solution.rho    = ia_mid(result.interv.rho);
solution.beta   = ia_mid(result.interv.beta);
solution.gamma  = ia_mid(result.interv.gamma);

if strcmpi(type,'ofbm')
    [ Ey11, Ey12, Ey22] = parametricModelBivOfBm( solution.h1, solution.h2, solution.rho, solution.sigma1, solution.sigma2, solution.beta, solution.gamma, j1, j2, Nwt, param.mu_bb );
else
    [ Ey11, Ey12, Ey22] = parametricModelBivOfGn( solution.h1, solution.h2, solution.rho, solution.sigma1, solution.sigma2, solution.beta, solution.gamma, j1, j2, Nwt, param.mu_bb );
end

S.y11 = Sy11;
S.y12 = Sy12;
S.y22 = Sy22;
E.y11 = Ey11;
E.y12 = Ey12;
E.y22 = Ey22;


%% Display
if param.dispFig > 0
    
    aaxis = [j1 j2 floor(log2(min([min(Sy11), min(abs(Sy12)), min(Sy22), min(Ey11), min(abs(Ey12)), min(Ey22)]))) ceil(log2(max([max(Sy11), max(abs(Sy12)), max(Sy22), max(Ey11), max(abs(Ey12)), max(Ey22)])))];
    
    figure(param.dispFig);clf;
    ax(1)=subplot(311);
    plot(j1:j2,log2(Sy11),'xk','markersize',20);hold on;
    plot(j1:j2,log2(Ey11),'-r','linewidth',2);
    xlabel('$j=\log_2$ scale','Interpreter','latex');
    l=legend('$\log_2 S_{11}(2^j)$','$\log_2 E_{11}(2^j,\widehat{\Theta})$');
    set(l,'Interpreter','latex','Location','NorthWest');
    axis(aaxis);
    set(gca,'fontsize',20);
    grid on;
    ax(2)=subplot(312);
    plot(j1:j2,log2(abs(Sy12)),'xk','markersize',20);hold on;
    plot(j1:j2,log2(abs(Ey12)),'-r','linewidth',2);
    xlabel('$j=\log_2$ scale','Interpreter','latex');
    l=legend('$\log_2 |S_{12}(2^j)|$','$\log_2 |E_{12}(2^j,\widehat{\Theta})|$');
    set(l,'Interpreter','latex','Location','NorthWest');
    axis(aaxis);
    set(gca,'fontsize',20);
    grid on
    ax(3)=subplot(313);
    plot(j1:j2,log2(Sy22),'xk','markersize',20);hold on;
    plot(j1:j2,log2(Ey22),'-r','linewidth',2);
    xlabel('$j=\log_2$ scale','Interpreter','latex');
    l=legend('$\log_2 S_{22}(2^j)$','$\log_2 E_{22}(2^j,\widehat{\Theta})$');
    set(l,'Interpreter','latex','Location','NorthWest');
    axis(aaxis);
    set(gca,'fontsize',20);
    grid on
    linkaxes(ax,'x');
    
end

end

