% (November 09, 2016)
%
% Author:
% Jordan Frecon (jordan.frecon@ens-lyon.fr) 
%
% Contributors:
% Gustavo Didier (gdidier@tulane.edu)
% Nelly Pustelnik (nelly.pustelnik@ens-lyon.fr)
% Patrice Abry (patrice.abry@ens-lyon.fr)
%
% This software is governed by the CeCILL license under French law and
% abiding by the rules of distribution of free software.  You can  use,
% modify and/ or redistribute the software under the terms of the CeCILL
% license as circulated by CEA, CNRS and INRIA at the following URL
% "http://www.cecill.info".
%
% As a counterpart to the access to the source code and  rights to copy,
% modify and redistribute granted by the license, users are provided only
% with a limited warranty  and the software's author,  the holder of the
% economic rights,  and the successive licensors  have only  limited
% liability.
%
% In this respect, the user's attention is drawn to the risks associated
% with loading,  using,  modifying and/or developing or reproducing the
% software by the user in light of its specific status of free software,
% that may mean  that it is complicated to manipulate,  and  that  also
% therefore means  that it is reserved for developers  and  experienced
% professionals having in-depth computer knowledge. Users are therefore
% encouraged to load and test the software's suitability as regards their
% requirements in conditions enabling the security of their systems and/or
% data to be ensured and,  more generally, to use and operate it in the
% same conditions as regards security.
%
% The fact that you are presently reading this means that you have had
% knowledge of the CeCILL license and that you accept its terms.
%
%--------------------------------------------------------------------------
% analyzeBivOfbm_BB: Non-linear wavelet regression and branch & bound
%   optimization for the full identification of bivariate operator
%   fractional Brownian motion (OfBm).
%
% Demo file for analyzeBivOfBm_BB.m
%
% For more information, type 'help analyzeBivOfBm_BB'
%
% For theoretical aspects please refer to :
% # J. Frecon, G. Didier, N. Pustelnik, and P. Abry.
% Non-linear wavelet regression and branch & bound optimization for the full identification of bivariate operator fractional Brownian motion.
% IEEE Trans. Signal Process., 64(15):4040-4049, 2016.
%
%%

clear all;
close all;
clc;

mydir  = which('analyzeBivOfBm_BB_demo.m');
idcs   = strfind(mydir,'/');
newdir = mydir(1:idcs(end));
addpath(genpath(newdir));



%% - PARAMETERS

type        = 'ofbm';   % Either 'ofbm' or 'ofgn'
N           = 2^18;     % Sample size


param.h1        = .6;   % Hurst parameters
param.h2        = .7;
param.sigma1    = 1;    % Variances of increments
param.sigma2    = 1;
param.rho       = .8;   % Correlation
param.beta      = .5;   % Mixing parameters 
param.gamma     = .2;



%% - SYNTHESIS
disp('** Synthesis')
disp(param)

[Y,X]       = synthesizeBivOfBm(type,N,param );



%% - ANALYSIS
disp('** Analysis')

% Optional: A priori information on parameters
% (interval or value)
clear interv
interv.h2       = [.5 .9];
interv.sigma1   = param.sigma1;
interv.sigma2   = param.sigma2;
interv.rho      = [0.5 1];
interv.gamma    = [.1 .3];

% Optional: Requested precision on parameters
clear prec
prec.h1 = 0.01;
prec.h2 = 0.01;

% Optional: Additional options
clear param
param.crossTerm = false;


tic
[paramEst,spectrEmp, spectrEst] = analyzeBivOfBm_BB( Y, type,interv,param,prec);
toc



%% - DISPLAY
disp('** Estimate:')
disp(paramEst)

figure(1);clf;
subplot(211);
plot(X');
grid on;
ylabel('Before mixing','Interpreter','latex');
title(strcat('Realization of$\;$',type),'Interpreter','latex');
l=legend('$X_1$','$X_2$');
set(l,'Interpreter','latex');
set(gca,'fontsize',15);
subplot(212);
plot(Y');
grid on;
ylabel('After mixing','Interpreter','latex');
l=legend('$Y_1$','$Y_2$');
set(l,'Interpreter','latex');
set(gca,'fontsize',15);

figure(2);clf;
subplot(311);
semilogy(spectrEmp.y11,'-ok','linewidth',2); hold on;
semilogy(spectrEst.y11,'--xr','linewidth',2);
xlabel('$\log_2$ scale','Interpreter','latex');
ylabel('1st Auto-component','Interpreter','latex');
title('Wavelet spectrum','Interpreter','latex');
l=legend('Empirical','Estimate');
set(l,'Interpreter','latex','Location','SouthEast');
set(gca,'fontsize',15);
grid on;
subplot(312);
semilogy(spectrEmp.y12,'-ok','linewidth',2); hold on;
semilogy(spectrEst.y12,'--xr','linewidth',2);
xlabel('$\log_2$ scale','Interpreter','latex');
ylabel('Cross-component','Interpreter','latex');
set(gca,'fontsize',15);
grid on;
subplot(313);
semilogy(spectrEmp.y22,'-ok','linewidth',2); hold on;
semilogy(spectrEst.y22,'--xr','linewidth',2);
xlabel('$\log_2$ scale','Interpreter','latex');
ylabel('2nd Auto-component','Interpreter','latex');
set(gca,'fontsize',15);
grid on;

