function R = CovarFARIMA0D0(N,d,W,Sigmae)
% Multivariate FARIMA(0,D,0) covariance generator
%
% Usage:
%   R = CovarFARIMA0D0(N,d,W,Sigmae)
% Inputs:
%   N     length of the covariance sequence to generate
%   d     row vector of length P, has to satisfy -1/2<d(k)<1/2, 
%         see "Comments" below
%   W     an invertible P-by-P matrix, see "Comments" below
%   Sigmae matrix of size P-by-P, see "Comments" below
% Output:
%   R     generated covariance structure, a cell structure of size P-by-P.
%
% Comments:
%   For the inputs above, D=W*diag(d)*inv(W).
%
%   Multivariate FARIMA(0,D,0) series are defined by
%     (I-B)^D X(n) = e(n)
%   where e(n) are i.i.d. Gaussian with mean zero and
%   covariance matrix E e(n)e(n)^T = Sigmae,
%   B is the backshift operator. Or,
%     X(n) = sum_{k=0}^\infty A_k e(n-k),
%   where the matrix A_k depends upon D.
%
% Copyright 2009, Hannes Helgason, ENS Lyon

P = length(d); % dimensionality

% ensure the elements in d satisfy -1/2<d(k)<1/2
ind = (d<=-1/2 | d>=1/2);
if sum(ind)>0,
    error('d does not satisfy -1/2<d(k)<1/2, for all k')
end
% ensure W has full rank
rankW = rank(W);
if rankW~=P,
    error('W does not have full rank');
end


% Calculate C
C = inv(W)*Sigmae*(inv(W)');

% Calculate R

% Variable storing covariance, RR(:,:,n+1)=EX_n X_0*
RR = zeros(P,P,N);

% PEND: vectorize these loops (?)
Gamma0 = zeros(P,P);
for n= 0:(N-1),
    for k=1:P,
        for j=1:P,
            num = C(k,j)*gammaComplex(1-d(k)-conj(d(j)));
            tmp = gammalnComplex(1+n-conj(d(j)))+gammalnComplex(1-n-d(k));
            denom = (-1)^n*exp(tmp);
            % Note: It is numerically unstable to calculate the
            %       denominator as
            %           denom = (-1)^n*gammaComplex(1+n-conj(d(j)))*...
            %               gammaComplex(1-n-d(k));
            %       when the arguments of the gamma functions grow --
            %       instead we work with the logarithm of the gamma function.

            % Forcing Gamma to be real-valued, ok since we assume d to be real-valued 
            num = real(num); denom = real(denom);
            Gamma0(k,j) = num/denom;
        end
    end
    RR(:,:,n+1) = W*Gamma0*(W');
end

% write the result in a cell array of size P-by-P

R = cell(P,P);
for k=1:P,
    for j=1:P,
        R{k,j} = squeeze(RR(k,j,:)).'; % take transpose to get a row vector
    end
end
