% Demo for Univariate non-Gaussian synthesis based on Hermite expansions

% The transform f for Y=f(X) is passed to the synthesis routine by a function handle
% define the function handle
mu = 0; sigma2 = 1; sigma = sqrt(sigma2); % parameters for transform
funch = @(x) exp(sigma*x + mu);
%funch = @(x) chi2inv(normcdf(x),1);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% FIRST WE SHOW HOW ONE CAN CALCULATE HERMITE EXPANSIONS AND COVARIANCE INVERSION USING
% THE CODE. AT THE END IS AN EXAMPLE FOR SYNTHESIS

% Calculate the Hermite expansion of the function
M = 30; % the highest order of Hermite polynomial in the expansion, 
        % i.e., f(x) \approx \sum_{m=0}^M coeffs(m+1) H_m(x)
coeffs = HermiteCoeffExpansion(funch,M);
x = linspace(-1,1,1000); % the points to evaluate the function at
% calculate the Hermite approximation
fHermiteApprox = HermiteExpansionEval(x,coeffs);

% Plot the function and the Hermite approximation
figure(1); clf;
plot(x,funch(x),x,fHermiteApprox,'--');
xlabel('x');ylabel('f or f^M');
legend('f : original function','f^M : M-order Hermite approximation','Location','Best');
title(sprintf('M = %d',M));


% Calculate the coefficients in the forward covariance relationship,
%   rY= \sum_{m=1}^M fwdSeq(m)*rX^m
% and in the reversion series, which can be used to invert the above relationship,
% i.e.,
%   rX \approx \sum_{m=1}^M revSeq(m)*rY^m
% NOTE! This series does not necessarily give the inverse for all rY, but for
% some neighborhood around rY=0
% NOTE: The current version of the code requires c_1~=0, i.e., coeffs(2)~=0
zeropadLength = 0; % use this to increase the number of terms in the reversion series 
[fwdSeq, revSeq] = CovarSequence(coeffs, coeffs, zeropadLength);

rX = linspace(-1,1,1000);
% calculate forward mapping
polyCoeff = fliplr([0 fwdSeq]); % setting constant term to 0 and flipping
rY = polyval(polyCoeff,rX);

% reversion
w = linspace(min(rY),max(rY),1000);
polyCoeff = fliplr([0 revSeq]); % setting constant term to 0 and flipping
rXreversion = polyval(polyCoeff,w);

figure(2); clf;
plot(rX,rY); hold on;
xaxislim = xlim; yaxislim = ylim;
plot(rXreversion,w,'r--');
axis([xaxislim yaxislim]);
xlabel('r^X'); ylabel('r^Y');
legend('Forward relationship','Approx. using series reversion','Location','Best');

% calculate the mapping g, by normalizing rY -- that is rhoY
% we get the variance by plugging rX=1
varY = polyval(fliplr([0 fwdSeq]),1); 
rhoY = rY/varY;

figure(3); clf;
plot(rX,rhoY); hold on;
plot(rXreversion,w/varY,'r--');
axis([-1 1 -1 1]);
xlabel('r^X'); ylabel('\rho^Y'); 
title(sprintf('The mapping of correlations'));
legend('Forward relationship','Approx. using series reversion','Location','Best');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% THE CODE BELOW IS WHAT MATTERS FOR SYNTHESIS
% length of synthesized series
N = 2^10; 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Generate covariance structure
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % set parameters for the FARIMA(0,D,0) model
W = 1; % this parameter is not needed for the univariate case
d = 0.3;
Sigmae = 1;

% generate targeted covariance structure
Ncov = N+1; % cov. could be asymmetric, get cov. seq. of length N+1
rY = CovarFARIMA0D0(Ncov,d,W,Sigmae);
% NOTE: rY might not be normalized in a proper way (matching the variance
% defined by funch) (...rYnormalized = varY/rY(1) * rY;)
% At the moment, GenUnivarNonGaussUsingHermite does it for you

% Synthesize non-Gaussian series using the 'best approximation' of the
% inverted covariance mapping
M = 30; % order of Hermite expansion
[Y,rYapprox,coeffs,X,rXapprox,BmArray] = GenUnivarNonGaussUsingHermite(rY,funch,N,M);


% The variable BmArray can be used for Monte Carlo simulations this way:
% > [X,Xcopy] = SynthStepMultivarGaussBestApprox(BmArray,N);
% > Y1 = funch(X); Y2 = funch(Xcopy);
% where Y1 and Y2 will be two independent realizations of Y
% To get B realizations:
% B = 1000;
% numIts = ceil(B/2); % rounding up to nearest integer of B/2
% Yarray = zeros(2*numIts,N); % init data array, one realization per row
% for k=1:numIts,
%   [X,Xcopy] = SynthStepMultivarGaussBestApprox(BmArray,N);
%   Yarray(2*k-1,:) = funch(X);
%   Yarray(2*k,:) = funch(Xcopy);
% end

% Plot difference of the targeted and resulting covariance
% Normalize
rY = varY/rY{1}(1)*rY{1}(1:N); % using "{}" since rY is a cell array
% plot the covariance sequences
nn = 0:(length(rY)-1);
figure;
subplot(3,1,1); plot(nn,rY); xlabel('n'); ylabel('r_Y');
subplot(3,1,2); semilogy(nn,abs(rY-rYapprox)); xlabel('n'); ylabel('r_Y - r_{Y,approx}');
subplot(3,1,3); plot(log10(nn),log10(abs(rY)),log10(nn),log10(abs(rYapprox)),'r--'); 
xlabel('log_{10} n'); ylabel('log_{10} |r[n]|'); legend('r_Y','r_{Y,approx}');

% plot a realization of the non-Gaussian series Y and Gaussian series X
figure;
subplot(2,1,1);plot(Y);xlabel('n');ylabel('Y[n]=f(X[n])'); title('Realization of non-Gaussian series Y[n]=f(X[n])');
subplot(2,1,2);plot(X);xlabel('n');ylabel('X[n]'); title('Gaussian series realization used in the transform');
