function [X,Xiid] = SynthStepMultivarGauss(sqrtLambda,crossSpec,N)
% Generate multivariate stationary Gaussian series,
% provided preprocessed information from InitMultivarGauss
%
% Usage:
% [X,Xiid] = SynthStepMultivarGauss(sqrtLambda,crossSpec,N)
%
% Inputs:
%   sqrtLambda, crossSpec, N  see documentation given in InitMultivarGauss
% Outputs:
%   X and Xiid are two independent realizations of the series, stored
%   as matrices of size P-by-N. X(k,:) is a realization of Xk, i.e., 
%   component k, and is a vector of length N. Same holds for Xiid.
%
% 
% Copyright (c) Hannes Helgason, ENS Lyon, 2009

% find the number of components, P, to generate
% and the length of the circularly embedded covariance sequences
[P,M] = size(sqrtLambda);

Z = zeros(P,M); % using a matrix structure to store the vectors Z1, Z2,...,ZP in rows
Cm = zeros(P,P); % init
for m=1:M,
  Cupper = zeros(P,P);
  % fill in upper-half of the matrix Cm
  for k=1:P,
    for j=(k+1):P,
      Cupper(k,j) = crossSpec{k,j}(m);
    end
  end
  % the lower-half of Cm is the conjugate transpose of its upper-half
  Cm = Cupper + Cupper';

  % fill in the diagonal entries
  for k=1:P,
    Cm(k,k) = 2;
  end

  % find the Cholesky factorization of Cm
  Am = chol(Cm,'lower'); % this produces a lower triangular matrix such that Cm=Am*Am'
  W = (randn(P,1) + i*randn(P,1))/sqrt(2);
  tmp = Am*W;
  Z(:,m) = tmp;
end

% Generate each component
tmp = sqrtLambda.*Z; % multiply with sqrtLambda to get correct auto-covariance
Xtilde = 1/sqrt(M)*fft(tmp,[],2); % taking DFT along rows
% Note: dividing by sqrt(M) to normalize correctly

% the first N row entries of the real and imaginary parts
% are independent realizations of the process (respectively)
X = real(Xtilde(:,1:N));
Xiid = imag(Xtilde(:,1:N));
