function [Y,RYapprox,approxFlag,BmArray] = GenMVnGaussUsingHermite(RY,funchArray,N,numCopies)
% Generates multivariate stationary non-Gaussian series based on mappings
% Y_p = f_p(X_p), where {X_p} is stationary multivariate Gaussian series.
% The method is based on Hermite expansions of the functions f_p.
%
% Usage:
%   [Ycopy1,Ycopy2,,RYapprox,K,funch] = GenMVnGaussUsingHermite(RY,funchArray,N,numCopies))
% Inputs:
%   RY      target covariance structure (see InitMVnGaussUsingHermite.m)
%   funchArray  cell array of function handles, funchArray{p} is the
%   transform f_p for the p-th component
%   N       length of the series to synthesize
%   numCopies  number of independent realizations of Y to synthesis
%           if omitted, numCopies = 1
% Outputs:
%   Y       Y(:,:,1),...,Ycopy1(:,:,numCopies) are numCopies
%           are indepedent realizations of the non-Gaussian multivariate 
%           series.
%           Y(p,:,k) is the p-th component in realization k
%   RYapprox    resulting covariance of the generated series. Not
%           necessarily the same as the targeted covariance RY
%   approxFlag  true if approximation was needed, otherwise false
%   BmArray     this structure can be passed to
%               SynthStepMultivarGaussBestApprox to generate samples of
%               the Gaussian series {X_p}
% 
% Copyright (c) 2010, Hannes Helgason, ENS de Lyon

if nargin < 4,
    % generate only one realization
    numCopies = 1;
end

[RYapprox,approxFlag,BmArray,coeffs,RXapprox] = InitMVnGaussUsingHermite(RY,funchArray,N);

P = size(RY,1); % number of components
Y = zeros(P,N,numCopies); % initialize output

numloops = floor(numCopies/2);
for k=1:numloops,
    [Xcopy1,Xcopy2] = SynthStepMultivarGaussBestApprox(BmArray,N);        
    % recall that Xcopy1 and Xcopy2 are independent copies of a
    % multivariate Gaussian series

    % Transform Gaussian series: Y_p=f_p(X_p)
    ind1 = 2*k-1;
    ind2 = 2*k;
    for p=1:P,
        Y(p,:,ind1) = funchArray{p}(Xcopy1(p,:));
    end
    
    for p=1:P,
        Y(p,:,ind2) = funchArray{p}(Xcopy2(p,:));
    end
end

if numCopies-2*numloops > 0,
    % numCopies is odd so add one more realization
    [Xcopy1] = SynthStepMultivarGaussBestApprox(BmArray,N);
    % Transform Gaussian series: Y_p=f_p(X_p)
    for p=1:P,
        Y(p,:,numCopies) = funchArray{p}(Xcopy1(p,:));
    end
end
