function plotCovMapHermite(funch1,funch2,plotCorr,plotReversion,numPts)
% Utility for plotting the covariance mapping rX -> rY, -1<=rX<=1, 
% or the numerical reversion of rY 
% (which gives rX in a neighborhood of zero but might diverge away from it):
%   rY = Cov(Y1,Y2) and rX = Cov(X1,X2) 
% with 
%   Y1=f1(X1) and Y2=f2(X2)
% for two functions f1 and f2 and X1, X2 bivariate N(0,1) variables.
% (The assumption about X1 and X2 imposes -1<=rX<=1)
%
% The calculation of the mapping utilizes Hermite expansions.
%
% Usage:
%   plotCovMapHermite(funch1,funch2,plotCorr,plotReversion,numPts)
%   plotCovMapHermite(funch1,funch2,plotCorr,plotReversion)
%   plotCovMapHermite(funch1,funch2,plotCorr)
%   plotCovMapHermite(funch1,funch2)
% Inputs:
%   funch1, funch2  function handles for the functions f1 and f2 above
%   plotCorr  set to 1 for plotting rhoX and rhoY 
%             (or the numerical reversion for rhoY)
%             where rhoX = Cov(X1,X2)/sqrt(Var(X1)Var(X2)) 
%             (similar definition for rhoY)
%             Default value is 0
%   plotReversion  set to 1 to plot the numerical reversion
%             Default value is 0
%   numPts    sets the resolution of the plot (for the forward mapping
%             rX -> rY, it uses rX = linspace(-1,1,numPts))
%             Default value is 1000
%
% See also DemoPlotCovMapHermite.m for a demo
%
% Copyright (c) 2010, Hannes Helgason, ENS de Lyon


if nargin < 5,
    numPts = 1000;
    if nargin < 4,
        plotReversion = 0;
        if nargin < 3,
            plotCorr = 0;
        end
    end
end

rX = linspace(-1,1,numPts);
rY = MapCovarHermite(rX,funch1,funch2,plotCorr);

if plotReversion,
    % calculate reversion
	w = linspace(min(rY),max(rY),numPts);
	calcReversion = 1;
    rXreversion = MapCovarHermite(w,funch1,funch2,plotCorr,calcReversion);
    plot(rXreversion,w,'r--');
else
    plot(rX,rY);
end