#include <mex.h>
/* Function for multiplying block-diagonal matrix diag([B1 B2 ... BM]),
 * (Bk of size P-by-P) a vector [W1^T W2^T ... Wm^T]^T
 * (Wk of column vector of length P). That is, calculates Bk*Wk
 *
 * Implementing this in mex is orders of magnitude faster than using a for
 * loop in Matlab (in the case M>>P, for example M=100,000, P=2)
 * 
 * Usage:
 *   U = BlockDiagMultiplication(B,W)
 * Inputs:
 *   B  matrix of size P-by-PM where
 *      B(:,(k-1)*P+(1:P)) stores block Bk (of size P-by-P)
 *   W  matrix of size P-by-M where
 *      W(:,k) stores the column vector Wk
 * Output:
 *   U  matrix of size P-by-M where U = [U1 U2 ... UM], with Uk=Bk*Wk
 *
 * Copyright (c) 2010, Hannes Helgason, ENS de Lyon
 */


/* Computational subroutines */
void blockmultiplyBothComplex( int P, int M, double *Br, double *Bi, 
             double *Wr, double *Wi, double *Ur, double *Ui) {
    int p,dummy,m;
    int indOffs, numColsB;
    double tmpr, tmpi;
    double br, bi, wr, wi;

    /* perform the matrix multiplications */
    indOffs = 0;
    for(m=0; m<M; m++) {
        for(p=0; p<P; p++) {
            tmpr = 0.0;
            tmpi = 0.0;            
            for(dummy=0; dummy<P; dummy++) {
                br = Br[(dummy+indOffs)*P + p];
                bi = Bi[(dummy+indOffs)*P + p];
                wr = Wr[dummy + m*P];
                wi = Wi[dummy + m*P];
                tmpr = tmpr + br*wr - bi*wi;
                tmpi = tmpi + br*wi + bi*wr;
            }
            Ur[p + m*P] = tmpr;
            Ui[p + m*P] = tmpi;            
        }
        indOffs = indOffs + P; 
    }
}

void blockmultiplyBComplex( int P, int M, double *Br, double *Bi, 
             double *Wr, double *Ur, double *Ui) {
    int p,dummy,m;
    int indOffs, numColsB;
    double tmpr, tmpi;
    double br, bi, wr;

    /* perform the matrix multiplications */
    indOffs = 0;
    for(m=0; m<M; m++) {
        for(p=0; p<P; p++) {
            tmpr = 0.0;
            tmpi = 0.0;            
            for(dummy=0; dummy<P; dummy++) {
                br = Br[(dummy+indOffs)*P + p];
                bi = Bi[(dummy+indOffs)*P + p];
                wr = Wr[dummy + m*P];
                tmpr = tmpr + br*wr;
                tmpi = tmpi + bi*wr;
            }
            Ur[p + m*P] = tmpr;
            Ui[p + m*P] = tmpi;            
        }
        indOffs = indOffs + P; 
    }
}

void blockmultiplyWComplex( int P, int M, double *Br, 
             double *Wr, double *Wi, double *Ur, double *Ui) {
    int p,dummy,m;
    int indOffs, numColsB;
    double tmpr, tmpi;
    double br, wr, wi;

    /* perform the matrix multiplications */
    indOffs = 0;
    for(m=0; m<M; m++) {
        for(p=0; p<P; p++) {
            tmpr = 0.0;
            tmpi = 0.0;            
            for(dummy=0; dummy<P; dummy++) {
                br = Br[(dummy+indOffs)*P + p];
                wr = Wr[dummy + m*P];
                wi = Wi[dummy + m*P];
                tmpr = tmpr + br*wr;
                tmpi = tmpi + br*wi;
            }
            Ur[p + m*P] = tmpr;
            Ui[p + m*P] = tmpi;            
        }
        indOffs = indOffs + P; 
    }
}


void blockmultiplyReal( int P, int M, double *Br, double *Wr, double *Ur ) {
    int p,dummy,m;
    int indOffs, numColsB;
    double tmp;
    
    /* perform the matrix multiplications */
    indOffs = 0;
    for(m=0; m<M; m++) {
        for(p=0; p<P; p++) {
            tmp = 0.0;
            for(dummy=0; dummy<P; dummy++) {
                tmp = tmp + Br[(dummy+indOffs)*P + p]*Wr[dummy + m*P];
            }
            Ur[p + m*P] = tmp;
        }
        indOffs = indOffs + P; 
    }
}



/* Mex-interface */
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {
    /* Declarations **/
    double *Br, *Bi; /* real and imaginary parts of B */
    double *Wr, *Wi; /* real and imaginary parts of W */
    double *Ur, *Ui; /* real and imaginary parts of U */
    int P, M;
    int nRowsB, nRowsW, nColsB, nColsW;
    
    /* Get input data */
    nRowsB = (int) mxGetM(prhs[0]);
    nColsB = (int) mxGetN(prhs[0]);
    nRowsW = (int) mxGetM(prhs[1]);
    nColsW = (int) mxGetN(prhs[1]);
    
    /* Check inputs */
	if(nrhs!=2) 
        mexErrMsgTxt("Two inputs required.");
	if (nRowsB != nRowsW || nColsB != nRowsW*nColsW)
        mexErrMsgTxt("The inputs matrices do not have matching dimensions. See documentation.");   
    
    /* Get pointers to real part of data */
    Br = mxGetPr(prhs[0]);
    Wr = mxGetPr(prhs[1]);
    
    /* Check whether inputs are complex-valued and call subroutines*/
    if ( mxIsComplex(prhs[0])) {
        /* B is complex */
        plhs[0] = mxCreateDoubleMatrix(nRowsW, nColsW, mxCOMPLEX);
        Ur = mxGetPr(plhs[0]);
        Ui = mxGetPi(plhs[0]);        
        Bi = mxGetPi(prhs[0]);
        if ( mxIsComplex(prhs[1])) {
            /* W is also complex */
            Wi = mxGetPi(prhs[1]);
            blockmultiplyBothComplex(nRowsW, nColsW, Br, Bi, Wr, Wi, Ur, Ui);
        } else {
            /* W is also real */
            blockmultiplyBComplex(nRowsW, nColsW, Br, Bi, Wr, Ur, Ui);
        }
    } else {
        if ( mxIsComplex(prhs[1])) {
            /* W is complex, B is not */
            plhs[0] = mxCreateDoubleMatrix(nRowsW, nColsW, mxCOMPLEX);
            Ur = mxGetPr(plhs[0]);
            Ui = mxGetPi(plhs[0]);
            Wi = mxGetPi(prhs[1]);
            blockmultiplyWComplex(nRowsW, nColsW, Br, Wr, Wi, Ur, Ui);
        } else {
            /* will use real multiplication */
            plhs[0] = mxCreateDoubleMatrix(nRowsW, nColsW, mxREAL);
            Ur = mxGetPr(plhs[0]);
            blockmultiplyReal(nRowsW, nColsW, Br, Wr, Ur);
        }
    }

    return;
}
