% (November 09, 2016)
%
% [ Y, X ] = synthesizeBivOfBm(type,nbsamples,param)
%
% Synthesize a realization of bivariate OfBm/OfGn
%
% Input:
%   - 'type' either 'ofbm' of 'ofgn'
%   - 'nbsamples' Number of samples
%   - 'param' Process parameters
%       - Hurst parameters ('param.h1', 'param.h2')
%       - Variance parameters ('param.sigma1', 'param.sigma2')
%       - Correlation ('param.rho')
%       - Mixing parameters ('param.beta','param.gamma')
%
% Output:
%   - 'Y' Realization after mixing
%   - 'X' Realizaton before mixing

% Dependency:
%   - InitStepMultivarGaussBestApprox.m          
%
function [ Y, X ] = synthesizeBivOfBm(type,nbsamples,param)



if param.rho^2 * gamma(param.h1+param.h2+1)^2 * sin(pi*(param.h1+param.h2)/2)^2 >= gamma(2*param.h1+1)*gamma(2*param.h2+1)*sin(pi*param.h1)*sin(pi*param.h2)
    disp('/!\ OfBm / OfGn is not correctly defined /!\');
    Y = NaN*ones(1,nbsamples);
    X = NaN*ones(1,nbsamples);
    
else
    
    
    % Parameters
    d1          = param.h1-.5;
    d2          = param.h2-.5;
    d12         = (d1+d2)/2 ;
    d           = [d1 d12 ; d12 d2] ;
    Sigmae      = [ param.sigma1^2 param.rho*param.sigma1*param.sigma2 ; param.rho*param.sigma1*param.sigma2 param.sigma2^2] ;
    W           = [1/sqrt(1+param.gamma^2) param.beta/sqrt(1+param.beta^2) ; -param.gamma/sqrt(1+param.gamma^2) 1/sqrt(1+param.beta^2)];
    Ncov        = nbsamples+1;
    h           = d+0.5;
    name        = 'Gaussian';
    params      = [0 2];
    TR          = 1;

    
    % - (1) - Initialisation
    Rx = CovarMfGn_sigma_v2(Ncov,[h(1,1) h(2,2)],Sigmae);
    [Rx,K,funch] = MapCovar(Rx,name,params);
    % NOTE: RX is not necessarily a valid covariance structure
    [Rxapprox,approxFlag,BmArray] = InitStepMultivarGaussBestApprox(Rx,nbsamples);
    if approxFlag,
        disp('Approximation was needed to ensure psd of inverted target covariance.');
    %else
    %    disp('Inverted target covariance was psd - no approximation needed.');
    end
    
    P           = length(Rx); % number of components in the series
    X_tauTmp    = cell(1,P);
    
    
    
    % - (2) - Generation of increments (gaussian, non-independant, stationnary) 
    for kk=1:floor(K/2),
        [Xcopy1,Xcopy2] = SynthStepMultivarGaussBestApprox(BmArray,nbsamples);
        % Xcopy1 and Xcopy2 are independent copies of the multivariate Gaussian process
        for p=1:P,
            X_tauTmp{p}(2*kk-1,:)  = Xcopy1(p,:);
            X_tauTmp{p}(2*kk,:)    = Xcopy2(p,:);
        end
    end
       
    if rem(K,2)>0,
        % add one more component since K is odd
        [Xcopy1,Xcopy2] = SynthStepMultivarGaussBestApprox(BmArray,nbsamples);
        for p=1:P,
            X_tauTmp{p}(K,:) = Xcopy1(p,:);
        end
    end
    
    % Transform Gaussian series
    for p=1:P,
        X_tau(p,:) = funch(X_tauTmp{p});
    end
    
    
    % - (3) -  Std
    X_tau(1,:)  = X_tau(1,:)./std(X_tau(1,:)) * param.sigma1 ;
    X_tau(2,:)  = X_tau(2,:)./std(X_tau(2,:)) * param.sigma2 ;
    
    
    % - (4) - Generation of motion = sum of increments
    X_1         = cumsum(X_tau(1,:)) ;
    X_2         = cumsum(X_tau(2,:)) ;
    X           = [X_1 ; X_2];
    time        = [0:1:length(X_1)-1]*TR ;
    
    
    
    % - (5) - Mixture
    Mix         = W*X;
    if strcmpi(type,'ofbm')
        Y       = Mix;
    else
        X       = diff(X,[],2);
        Y       = diff(Mix,[],2);
    end




end
if param.h1 > param.h2
    disp('/!\ Issue: please respect the convention param.h1 > param.h2 /!\')
end













end

